package Lingua::Stem::Snowball;

use strict;
use Carp;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD);

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA = qw(Exporter DynaLoader);
@EXPORT = qw(SnowballStemmers snowball);

use locale;   
use POSIX qw(locale_h);

$VERSION = '0.04';

sub AUTOLOAD {
    # This AUTOLOAD is used to 'autoload' constants from the constant()
    # XS function.  If a constant is not found then control is passed
    # to the AUTOLOAD in AutoLoader.

    my $constname;
    ($constname = $AUTOLOAD) =~ s/.*:://;
    croak "& not defined" if $constname eq 'constant';
    my $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
        if ($! =~ /Invalid/) {
            $AutoLoader::AUTOLOAD = $AUTOLOAD;
            goto &AutoLoader::AUTOLOAD;
        }
        else {
                croak "Your vendor has not defined Lingua::Stem::Snowball macro $constname";
        }
    }
    no strict 'refs';
    *$AUTOLOAD = sub () { $val };
    goto &$AUTOLOAD;
}

my %encodings = (
	russian	 => 'KOI8_R',
	_default => 'ISO_8859_1',
);

sub new {
        my ($class, %opt)=@_;
        $class = ref($class) || $class;
        my $self={};

	die "No lang option" if ( ! length $opt{lang} ); 
	$self->{LANG} = lc($opt{lang});

	$self->{LOCALE} = undef;
	if ( defined $opt{locale} ) {
		$self->{LOCALE} = $opt{locale};

		my $encoding = $opt{locale};
		$encoding =~ s/^.*\.//;
		$encoding =~ s/^ISO(\d)/ISO_$1/;
		$encoding =~ s/-/_/g;

		$self->{ENCODING} = $encoding;
	} elsif ( exists $encodings{ $self->{LANG} } ) {
		$self->{ENCODING} = $encodings{ $self->{LANG} };
	} else {
		$self->{ENCODING} = $encodings{ _default };
	}

	$self->{OBJ} = Lingua::Stem::Snowball::stem_init( $self->{LANG}, $self->{ENCODING} );

	die "Unknown lang or locale(encoding)" if ( !$self->{OBJ} );
 
        bless ($self, $class);

        return $self;
}

sub DESTROY {
	my $self=shift;
	Lingua::Stem::Snowball::stem_close( $self->{OBJ} );
}

sub stem {
	my ($self,$word, $rr) = @_;

	return undef if ! length $word;

	my $old_locale;
	if ( defined $self->{LOCALE} ) {
		$old_locale = setlocale(LC_CTYPE);
		my $ret = setlocale(LC_CTYPE, $self->{LOCALE});
		die "Can't set locale $self->{LOCALE}" if ! defined $ret;
	}

	my $lexem;
	my $res = Lingua::Stem::Snowball::do_stem($self->{OBJ}, $word, $lexem );
	if ( $res < 0 ) {
		die "Error in Lingua::Stem::Snowball::do_stem";
	}

	if ( defined $self->{LOCALE} ) {
		setlocale(LC_CTYPE, $old_locale);
	}
	$$rr = $res if ( ref $rr );

	return $lexem; 
}

sub SnowballStemmers {
	my @lang;
	Lingua::Stem::Snowball::get_stemmer_list( \@lang );
	return @lang;
}

sub snowball {
	my ($lang, $word, $locale, $rr) = @_;

	return Lingua::Stem::Snowball->new( lang=>$lang, locale=>$locale )->stem( $word, $rr );	
}

bootstrap Lingua::Stem::Snowball $VERSION;

1;

__END__

=head1 NAME

Lingua::Stem::Snowball - Perl interface to Snowball stemmers. 

=head1 PREREQUISITIES

Snowball - http://snowball.sourceforge.net/

=head1 SYNOPSIS

use  Lingua::Stem::Snowball;

OO interface:

my $lang = 'english';

my $dict=Lingua::Stem::Snowball->new( lang=>$lang );

my $locale = 'C'; 

my $dict=Lingua::Stem::Snowball->new( lang=>$lang, locale=>$locale );

my $lemm = $dict->stem( $word );

my $lemm = $dict->stem( $word, \$is_stemmed );

Plain interface:

my @lang = SnowballStemmers();

my $lemm = snowball( $lang, $word );

my $lemm = snowball( $lang, $word, $locale );

my $lemm = snowball( $lang, $word, $locale, \$is_stemmed );

=head1 DESCRIPTION

See README.

=head1 AUTHOR

Teodor Sigaev,  teodor@stack.net

=head1 SEE ALSO

http://snowball.tartarus.org/

=cut


